# -*- coding: utf-8 -*-
"""Генератор кода для библиотеки `comm_port'"""

import cog
import xml.etree.ElementTree as et

def getopt(xmlconf):
	"""Извлекает необходимые для генерации данные
	из файла настроек проекта"""
	try:
		xml = et.parse(xmlconf)
	except:
		cog.error('XMLConf: can\'t open file: %s' % xmlconf)
	# std -----------------------------------------------------------------
	try:
		mcu = xml.find('/mcu').attrib['value']
	except:
		cog.error('XMLConf: in\n<project>\n\t<mcu value=""/>')
	try:
		xtal = float(xml.find('/xtal').attrib['value'])
	except:
		cog.error('XMLConf: in\n<project>\n\t<xtal value=""/>')

	# comm_port -----------------------------------------------------------
	try:
		comm_port = xml.find('/comm_port')
		try:
			port = comm_port.find('./port')
			try:
				portid = int(port.attrib['id'])
			except:
				cog.error('XMLConf: in\n<project>\n\t<comm_port>\n\t\t<port id=""/>')
			try:
				baud = float(port.attrib['baud'])
			except:
				cog.error('XMLConf: in\n<project>\n\t<comm_port>\n\t\t<port baud=""/>')
			try:
				mode = port.attrib['mode']
			except:
				cog.error('XMLConf: in\n<project>\n\t<comm_port>\n\t\t<port mode=""/>')
			try:
				rxen = int(port.attrib['rx'])
			except:
				rxen = 1
			try:
				txen = int(port.attrib['tx'])
			except:
				txen = 1
			try:
				u2x = port.attrib['u2x']
			except:
				u2x = 'auto'
		except:
			cog.error('XMLConf: in\n<project>\n\t<comm_port>\n\t\t<port />')
		try:
			buff = comm_port.find('./buffer')
			try:
				rxbuf = int(buff.attrib['rx'])
			except:
				cog.error('XMLConf: in\n<project>\n\t<comm_port>\n\t\t<buffer rx=""/>')
			try:
				txbuf = int(buff.attrib['tx'])
			except:
				cog.error('XMLConf: in\n<project>\n\t<comm_port>\n\t\t<buffer tx=""/>')
		except:
			cog.error('XMLConf: in\n<project>\n\t<comm_port>\n\t\t<buffer />')
	except:
		cog.error('XMLConf: in\n<project>\n\t<comm_port>')

	return ((xtal, mcu), (portid, baud, u2x, mode), (txen, rxen), (txbuf, rxbuf), )

def out(xmlconf):
	"""Генератор кода"""
	((xtal, mcu), (port, baud, u2x, mode), (txen, rxen), (txbuf, rxbuf), ) = getopt(xmlconf)

	(ubrr, real_baud, baud_error, u2x) = calc_baud(xtal, baud, u2x)
	(ucsra, ucsrb, ucsrc) = calc_mode(mode, mcu, rxen, txen, u2x)
	
	if baud_error != 0 and baud_error < 2:
		cog.msg( "WARNING: baudrate error: %s%%" % baud_error )
	elif baud_error > 2:
		cog.error( "baudrate error: %s%%" % baud_error )

	cog.outl( "// COM conf: %s,%s x%s" %  (baud, mode, u2x+1) )

	if rxen:
		cog.outl( "#define PC_RX_BUF %s" %  rxbuf )
	if txen:
		cog.outl( "#define PC_TX_BUF %s" %  txbuf )
	cog.outl( '' )
	
	cog.outl( "#define PC_RX_EN %s" %  rxen )
	cog.outl( "#define PC_TX_EN %s\n" %  txen )

	cog.outl( "#define PC_UDR UDR%s" % port )
	cog.outl( "#define PC_UBRRH UBRR%sH" % port )
	cog.outl( "#define PC_UBRRL UBRR%sL" % port )
	cog.outl( "#define PC_UCSRA UCSR%sA" % port )
	cog.outl( "#define PC_UCSRB UCSR%sB" % port )
	cog.outl( "#define PC_UCSRC UCSR%sC\n" % port )

	if rxen:
		cog.outl( "#define PC_RX_VECT USART%s_RX_vect" %  port )
	if txen:
		cog.outl( "#define PC_UDRE_VECT USART%s_UDRE_vect" %  port )
		cog.outl( "#define PC_TX_VECT USART%s_TX_vect" %  port )
	cog.outl( '' )

	cog.outl( "#define PC_BAUD %s" % int(baud) )
	cog.outl( "#define PC_REAL_BAUD %s" %  real_baud )
	cog.outl( "#define PC_BAUD_ERR %s\n" %  baud_error )
	
	cog.outl( "#define PC_UBRRV %s" %  ubrr )
	cog.outl( "#define PC_UBRRHV %s" % (ubrr>>8) )
	cog.outl( "#define PC_UBRRLV %s" % (0xff & ubrr) )
	
	if (ucsra):
		cog.outl( "#define PC_UCSRAV %s" % ucsra )
	if (ucsrb):
		cog.outl( "#define PC_UCSRBV %s" % ucsrb )
	if (ucsrc):
		cog.outl( "#define PC_UCSRCV %s" % ucsrc )

def calc_baud(f_cpu, baud, u2x="auto"):
	"""Вычисляет UBRR, автоподбор U2X"""
	try:
		ubrr1 = int((f_cpu/(baud*16.0)) - 1.0) & 0x0fff
		real_baud1 = int(f_cpu / float(16.0 * (ubrr1 + 1.0)))
		baud_error1 = (float(real_baud1)/float(baud) - 1.0) * 100.0
	except:
		real_baud1 = False
	try:
		ubrr2 = int((f_cpu/(baud*8.0)) - 1.0) & 0x0fff
		real_baud2 = int(f_cpu / float(8.0 * (ubrr2 + 1.0)))
		baud_error2 = (float(real_baud2)/float(baud) - 1.0) * 100.0
	except:
		real_baud2 = False

	if u2x == 'auto':
		if real_baud1 or real_baud2:
			if baud_error1 <= baud_error2:
				u2x = 0
				ubrr = ubrr1
				real_baud = real_baud1
				baud_error = baud_error1
			else:
				u2x = 1
				ubrr = ubrr2
				real_baud = real_baud2
				baud_error = baud_error2
		elif real_baud1:
			u2x = 0
			ubrr = ubrr1
			real_baud = real_baud1
			baud_error = baud_error1
		elif real_baud2:
			u2x = 1
			ubrr = ubrr2
			real_baud = real_baud2
			baud_error = baud_error2
		else:
			cog.error('Baudrate error: can\'t calculate UBRR value')
	elif u2x == '0':
		u2x = 0
		ubrr = ubrr1
		real_baud = real_baud1
		baud_error = baud_error1
	elif u2x == '1':
		u2x = 1
		ubrr = ubrr2
		real_baud = real_baud2
		baud_error = baud_error2
	else:
		cog.error('XMLConf: in\n<project>\n\t<comm_port>\n\t\t<port u2x=""/>\n value error, must be: 0, 1, auto')

	return (ubrr, real_baud, baud_error, u2x)

def calc_mode(mode, chip, rxen=1, txen=1, u2x=0):
	"""Генеринует настройки битов регистров управления
	для работы порта в определенном режиме"""
	(bit, parity, stop) = mode

	if( bit == '9' ):
		(ucsz2, ucsz1, ucsz0) = (1, 1, 1)
	elif( bit == '8' ):
		(ucsz2, ucsz1, ucsz0) = (0, 1, 1)
	elif( bit == '7' ):
		(ucsz2, ucsz1, ucsz0) = (0, 1, 0)
	elif( bit == '6' ):
		(ucsz2, ucsz1, ucsz0) = (0, 0, 1)
	elif( bit == '5' ):
		(ucsz2, ucsz1, ucsz0) = (0, 0, 0)
	else:
		cog.error('XMLConf: in\n<project>\n\t<comm_port>\n\t\t<port mode="">\nBit\'s per frame must be: 5..9')

	if(parity == 'N'):
		(upm1, upm0) = (0, 0)
	elif(parity == 'E'):
		(upm1, upm0) = (1, 0)
	elif(parity == 'O'):
		(upm1, upm0) = (1, 1)
	else:
		cog.error('XMLConf: in\n<project>\n\t<comm_port>\n\t\t<port mode="">\nParity must be: N (none), E (even), O (odd)')

	if(stop == '1'):
		usbs = 0
	elif(stop == '2'):
		usbs = 1
	else:
		cog.error('XMLConf: in\n<project>\n\t<comm_port>\n\t\t<port mode="">\nStop bit(s) must be: 1 or 2')

	ucsra = '(1<<RXC)|(1<<TXC)|(1<<UDRE)|(%s<<U2X)' % (u2x)
	ucsrb = '(%s<<RXCIE)|(%s<<TXCIE)|(%s<<UDRIE)|(%s<<RXEN)|(%s<<TXEN)|(%s<<UCSZ2)' % (rxen, txen, txen, rxen, txen, ucsz2)
	ucsrc = '(1<<URSEL)|(0<<UMSEL)|(%s<<UPM1)|(%s<<UPM0)|(%s<<USBS)|(%s<<UCSZ1)|(%s<<UCSZ0)' % (upm1, upm0, usbs, ucsz1, ucsz0)
	
	return (ucsra, ucsrb, ucsrc)
